@@ -2,10 +2,10 @@
  * outfuncs.c
  *	  routines to convert a node to ascii representation
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- *	$Header: /cvsroot/pgsql/src/backend/nodes/outfuncs.c,v 1.147 2001/10/25 14:08:11 tgl Exp $
+ *	$Header: /cvsroot/pgsql-server/src/backend/nodes/outfuncs.c,v 1.176 2002/10/14 22:14:34 tgl Exp $
  *
  * NOTES
  *	  Every (plan) node in POSTGRES has an associated "out" routine which
@@ -105,20 +105,19 @@ _outOidList(StringInfo str, List *list)
 static void
 _outCreateStmt(StringInfo str, CreateStmt *node)
 {
-	appendStringInfo(str, " CREATE :relname ");
-	_outToken(str, node->relname);
+	appendStringInfo(str, " CREATE :relation ");
+	_outNode(str, node->relation);
 
-	appendStringInfo(str, "	:columns ");
+	appendStringInfo(str, "	:tableElts ");
 	_outNode(str, node->tableElts);
 
-	appendStringInfo(str, " :inhRelnames ");
-	_outNode(str, node->inhRelnames);
+	appendStringInfo(str, " :inhRelations ");
+	_outNode(str, node->inhRelations);
 
 	appendStringInfo(str, " :constraints ");
 	_outNode(str, node->constraints);
 
-	appendStringInfo(str, " :istemp %s :hasoids %s ",
-					 booltostr(node->istemp),
+	appendStringInfo(str, " :hasoids %s ",
 					 booltostr(node->hasoids));
 }
 
@@ -127,8 +126,8 @@ _outIndexStmt(StringInfo str, IndexStmt *node)
 {
 	appendStringInfo(str, " INDEX :idxname ");
 	_outToken(str, node->idxname);
-	appendStringInfo(str, " :relname ");
-	_outToken(str, node->relname);
+	appendStringInfo(str, " :relation ");
+	_outNode(str, node->relation);
 	appendStringInfo(str, " :accessMethod ");
 	_outToken(str, node->accessMethod);
 	appendStringInfo(str, " :indexParams ");
@@ -137,24 +136,32 @@ _outIndexStmt(StringInfo str, IndexStmt *node)
 	_outNode(str, node->whereClause);
 	appendStringInfo(str, " :rangetable ");
 	_outNode(str, node->rangetable);
-	appendStringInfo(str, " :unique %s :primary %s ",
+	appendStringInfo(str, " :unique %s :primary %s :isconstraint %s ",
 					 booltostr(node->unique),
-					 booltostr(node->primary));
+					 booltostr(node->primary),
+					 booltostr(node->isconstraint));
+}
+
+static void
+_outNotifyStmt(StringInfo str, NotifyStmt *node)
+{
+	appendStringInfo(str, " NOTIFY :relation ");
+	_outNode(str, node->relation);
 }
 
 static void
 _outSelectStmt(StringInfo str, SelectStmt *node)
 {
 	/* XXX this is pretty durn incomplete */
-	appendStringInfo(str, "SELECT :where ");
+	appendStringInfo(str, " SELECT :where ");
 	_outNode(str, node->whereClause);
 }
 
 static void
 _outFuncCall(StringInfo str, FuncCall *node)
 {
-	appendStringInfo(str, "FUNCTION ");
-	_outToken(str, node->funcname);
+	appendStringInfo(str, " FUNCCALL ");
+	_outNode(str, node->funcname);
 	appendStringInfo(str, " :args ");
 	_outNode(str, node->args);
 	appendStringInfo(str, " :agg_star %s :agg_distinct %s ",
@@ -169,23 +176,30 @@ _outColumnDef(StringInfo str, ColumnDef *node)
 	_outToken(str, node->colname);
 	appendStringInfo(str, " :typename ");
 	_outNode(str, node->typename);
-	appendStringInfo(str, " :is_not_null %s :raw_default ",
+	appendStringInfo(str, " :inhcount %d :is_local %s :is_not_null %s :raw_default ",
+					 node->inhcount,
+					 booltostr(node->is_local),
 					 booltostr(node->is_not_null));
 	_outNode(str, node->raw_default);
 	appendStringInfo(str, " :cooked_default ");
 	_outToken(str, node->cooked_default);
 	appendStringInfo(str, " :constraints ");
 	_outNode(str, node->constraints);
+	appendStringInfo(str, " :support ");
+	_outNode(str, node->support);
 }
 
 static void
 _outTypeName(StringInfo str, TypeName *node)
 {
-	appendStringInfo(str, " TYPENAME :name ");
-	_outToken(str, node->name);
-	appendStringInfo(str, " :timezone %s :setof %s typmod %d :arrayBounds ",
+	appendStringInfo(str, " TYPENAME :names ");
+	_outNode(str, node->names);
+	appendStringInfo(str, " :typeid %u :timezone %s :setof %s"
+					 " :pct_type %s :typmod %d :arrayBounds ",
+					 node->typeid,
 					 booltostr(node->timezone),
 					 booltostr(node->setof),
+					 booltostr(node->pct_type),
 					 node->typmod);
 	_outNode(str, node->arrayBounds);
 }
@@ -204,62 +218,52 @@ _outIndexElem(StringInfo str, IndexElem *node)
 {
 	appendStringInfo(str, " INDEXELEM :name ");
 	_outToken(str, node->name);
+	appendStringInfo(str, " :funcname ");
+	_outNode(str, node->funcname);
 	appendStringInfo(str, " :args ");
 	_outNode(str, node->args);
-	appendStringInfo(str, " :class ");
-	_outToken(str, node->class);
+	appendStringInfo(str, " :opclass ");
+	_outNode(str, node->opclass);
 }
 
 static void
 _outQuery(StringInfo str, Query *node)
 {
+	appendStringInfo(str, " QUERY :command %d :source %d :utility ",
+					 (int) node->commandType, (int) node->querySource);
 
-	appendStringInfo(str, " QUERY :command %d ", node->commandType);
-
+	/*
+	 * Hack to work around missing outfuncs routines for a lot of the
+	 * utility-statement node types.  (The only one we actually *need* for
+	 * rules support is NotifyStmt.)  Someday we ought to support 'em all,
+	 * but for the meantime do this to avoid getting lots of warnings when
+	 * running with debug_print_parse on.
+	 */
 	if (node->utilityStmt)
 	{
-		/*
-		 * Hack to make up for lack of outfuncs for utility-stmt nodes
-		 */
 		switch (nodeTag(node->utilityStmt))
 		{
 			case T_CreateStmt:
-				appendStringInfo(str, " :create ");
-				_outToken(str, ((CreateStmt *) (node->utilityStmt))->relname);
-				appendStringInfo(str, " ");
-				_outNode(str, node->utilityStmt);
-				break;
-
 			case T_IndexStmt:
-				appendStringInfo(str, " :index ");
-				_outToken(str, ((IndexStmt *) (node->utilityStmt))->idxname);
-				appendStringInfo(str, " on ");
-				_outToken(str, ((IndexStmt *) (node->utilityStmt))->relname);
-				appendStringInfo(str, " ");
+			case T_NotifyStmt:
 				_outNode(str, node->utilityStmt);
 				break;
-
-			case T_NotifyStmt:
-				appendStringInfo(str, " :notify ");
-				_outToken(str, ((NotifyStmt *) (node->utilityStmt))->relname);
-				break;
-
 			default:
-				appendStringInfo(str, " :utility ? ");
+				appendStringInfo(str, "?");
+				break;
 		}
 	}
 	else
-		appendStringInfo(str, " :utility <>");
+		appendStringInfo(str, "<>");
 
 	appendStringInfo(str, " :resultRelation %d :into ",
 					 node->resultRelation);
-	_outToken(str, node->into);
+	_outNode(str, node->into);
 
-	appendStringInfo(str, " :isPortal %s :isBinary %s :isTemp %s"
+	appendStringInfo(str, " :isPortal %s :isBinary %s"
 					 " :hasAggs %s :hasSubLinks %s :rtable ",
 					 booltostr(node->isPortal),
 					 booltostr(node->isBinary),
-					 booltostr(node->isTemp),
 					 booltostr(node->hasAggs),
 					 booltostr(node->hasSubLinks));
 	_outNode(str, node->rtable);
@@ -296,6 +300,8 @@ _outQuery(StringInfo str, Query *node)
 
 	appendStringInfo(str, " :resultRelations ");
 	_outIntList(str, node->resultRelations);
+
+	/* planner-internal fields are not written out */
 }
 
 static void
@@ -553,6 +559,18 @@ _outSubqueryScan(StringInfo str, SubqueryScan *node)
 	_outNode(str, node->subplan);
 }
 
+/*
+ *	FunctionScan is a subclass of Scan
+ */
+static void
+_outFunctionScan(StringInfo str, FunctionScan *node)
+{
+	appendStringInfo(str, " FUNCTIONSCAN ");
+	_outPlanInfo(str, (Plan *) node);
+
+	appendStringInfo(str, " :scanrelid %u ", node->scan.scanrelid);
+}
+
 /*
  *	Material is a subclass of Plan
  */
@@ -709,6 +727,9 @@ _outExpr(StringInfo str, Expr *node)
 		case OP_EXPR:
 			opstr = "op";
 			break;
+		case DISTINCT_EXPR:
+			opstr = "distinct";
+			break;
 		case FUNC_EXPR:
 			opstr = "func";
 			break;
@@ -779,10 +800,8 @@ _outConst(StringInfo str, Const *node)
 static void
 _outAggref(StringInfo str, Aggref *node)
 {
-	appendStringInfo(str, " AGGREG :aggname ");
-	_outToken(str, node->aggname);
-	appendStringInfo(str, " :basetype %u :aggtype %u :target ",
-					 node->basetype, node->aggtype);
+	appendStringInfo(str, " AGGREG :aggfnoid %u :aggtype %u :target ",
+					 node->aggfnoid, node->aggtype);
 	_outNode(str, node->target);
 
 	appendStringInfo(str, " :aggstar %s :aggdistinct %s ",
@@ -817,16 +836,18 @@ static void
 _outArrayRef(StringInfo str, ArrayRef *node)
 {
 	appendStringInfo(str,
-		" ARRAYREF :refelemtype %u :refattrlength %d :refelemlength %d ",
-					 node->refelemtype,
+		 " ARRAYREF :refrestype %u :refattrlength %d :refelemlength %d ",
+					 node->refrestype,
 					 node->refattrlength,
 					 node->refelemlength);
 
-	appendStringInfo(str, " :refelembyval %c :refupperindex ",
-					 node->refelembyval ? 't' : 'f');
+	appendStringInfo(str,
+				   ":refelembyval %s :refelemalign %c :refupperindexpr ",
+					 booltostr(node->refelembyval),
+					 node->refelemalign);
 	_outNode(str, node->refupperindexpr);
 
-	appendStringInfo(str, " :reflowerindex ");
+	appendStringInfo(str, " :reflowerindexpr ");
 	_outNode(str, node->reflowerindexpr);
 
 	appendStringInfo(str, " :refexpr ");
@@ -842,9 +863,12 @@ _outArrayRef(StringInfo str, ArrayRef *node)
 static void
 _outFunc(StringInfo str, Func *node)
 {
-	appendStringInfo(str, " FUNC :funcid %u :functype %u ",
+	appendStringInfo(str,
+		" FUNC :funcid %u :funcresulttype %u :funcretset %s :funcformat %d ",
 					 node->funcid,
-					 node->functype);
+					 node->funcresulttype,
+					 booltostr(node->funcretset),
+					 (int) node->funcformat);
 }
 
 /*
@@ -854,10 +878,11 @@ static void
 _outOper(StringInfo str, Oper *node)
 {
 	appendStringInfo(str,
-					 " OPER :opno %u :opid %u :opresulttype %u ",
+				" OPER :opno %u :opid %u :opresulttype %u :opretset %s ",
 					 node->opno,
 					 node->opid,
-					 node->opresulttype);
+					 node->opresulttype,
+					 booltostr(node->opretset));
 }
 
 /*
@@ -894,9 +919,11 @@ _outRelabelType(StringInfo str, RelabelType *node)
 {
 	appendStringInfo(str, " RELABELTYPE :arg ");
 	_outNode(str, node->arg);
-
-	appendStringInfo(str, " :resulttype %u :resulttypmod %d ",
-					 node->resulttype, node->resulttypmod);
+	appendStringInfo(str,
+					 " :resulttype %u :resulttypmod %d :relabelformat %d ",
+					 node->resulttype,
+					 node->resulttypmod,
+					 (int) node->relabelformat);
 }
 
 /*
@@ -939,10 +966,7 @@ _outJoinExpr(StringInfo str, JoinExpr *node)
 	_outNode(str, node->quals);
 	appendStringInfo(str, " :alias ");
 	_outNode(str, node->alias);
-	appendStringInfo(str, " :colnames ");
-	_outNode(str, node->colnames);
-	appendStringInfo(str, " :colvars ");
-	_outNode(str, node->colvars);
+	appendStringInfo(str, " :rtindex %d ", node->rtindex);
 }
 
 /*
@@ -958,19 +982,50 @@ _outTargetEntry(StringInfo str, TargetEntry *node)
 	_outNode(str, node->expr);
 }
 
+static void
+_outAlias(StringInfo str, Alias *node)
+{
+	appendStringInfo(str, " ALIAS :aliasname ");
+	_outToken(str, node->aliasname);
+	appendStringInfo(str, " :colnames ");
+	_outNode(str, node->colnames);
+}
+
 static void
 _outRangeTblEntry(StringInfo str, RangeTblEntry *node)
 {
-	appendStringInfo(str, " RTE :relname ");
-	_outToken(str, node->relname);
-	appendStringInfo(str, " :relid %u ",
-					 node->relid);
-	appendStringInfo(str, " :subquery ");
-	_outNode(str, node->subquery);
-	appendStringInfo(str, " :alias ");
+	/* put alias + eref first to make dump more legible */
+	appendStringInfo(str, " RTE :alias ");
 	_outNode(str, node->alias);
 	appendStringInfo(str, " :eref ");
 	_outNode(str, node->eref);
+	appendStringInfo(str, " :rtekind %d ",
+					 (int) node->rtekind);
+	switch (node->rtekind)
+	{
+		case RTE_RELATION:
+		case RTE_SPECIAL:
+			appendStringInfo(str, ":relid %u", node->relid);
+			break;
+		case RTE_SUBQUERY:
+			appendStringInfo(str, ":subquery ");
+			_outNode(str, node->subquery);
+			break;
+		case RTE_FUNCTION:
+			appendStringInfo(str, ":funcexpr ");
+			_outNode(str, node->funcexpr);
+			appendStringInfo(str, " :coldeflist ");
+			_outNode(str, node->coldeflist);
+			break;
+		case RTE_JOIN:
+			appendStringInfo(str, ":jointype %d :joinaliasvars ",
+							 (int) node->jointype);
+			_outNode(str, node->joinaliasvars);
+			break;
+		default:
+			elog(ERROR, "bogus rte kind %d", (int) node->rtekind);
+			break;
+	}
 	appendStringInfo(str, " :inh %s :inFromCl %s :checkForRead %s"
 					 " :checkForWrite %s :checkAsUser %u",
 					 booltostr(node->inh),
@@ -1212,31 +1267,6 @@ _outDatum(StringInfo str, Datum value, int typlen, bool typbyval)
 	}
 }
 
-static void
-_outIter(StringInfo str, Iter *node)
-{
-	appendStringInfo(str, " ITER :iterexpr ");
-	_outNode(str, node->iterexpr);
-}
-
-static void
-_outStream(StringInfo str, Stream *node)
-{
-	appendStringInfo(str,
-	  " STREAM :pathptr @ %p :cinfo @ %p :clausetype %p :upstream @ %p ",
-					 node->pathptr,
-					 node->cinfo,
-					 node->clausetype,
-					 node->upstream);
-
-	appendStringInfo(str,
-			 " :downstream @ %p :groupup %d :groupcost %f :groupsel %f ",
-					 node->downstream,
-					 node->groupup,
-					 node->groupcost,
-					 node->groupsel);
-}
-
 static void
 _outAExpr(StringInfo str, A_Expr *node)
 {
@@ -1253,7 +1283,7 @@ _outAExpr(StringInfo str, A_Expr *node)
 			appendStringInfo(str, "NOT ");
 			break;
 		case OP:
-			_outToken(str, node->opname);
+			_outNode(str, node->name);
 			appendStringInfo(str, " ");
 			break;
 		default:
@@ -1291,26 +1321,47 @@ _outValue(StringInfo str, Value *value)
 			appendStringInfo(str, " %s ", value->val.str);
 			break;
 		default:
-			elog(NOTICE, "_outValue: don't know how to print type %d ",
+			elog(WARNING, "_outValue: don't know how to print type %d ",
 				 value->type);
 			break;
 	}
 }
 
 static void
-_outIdent(StringInfo str, Ident *node)
+_outRangeVar(StringInfo str, RangeVar *node)
 {
-	appendStringInfo(str, " IDENT ");
-	_outToken(str, node->name);
+	appendStringInfo(str, " RANGEVAR :relation ");
+
+	/*
+	 * we deliberately ignore catalogname here, since it is presently not
+	 * semantically meaningful
+	 */
+	_outToken(str, node->schemaname);
+	appendStringInfo(str, " . ");
+	_outToken(str, node->relname);
+	appendStringInfo(str, " :inhopt %d :istemp %s",
+					 (int) node->inhOpt,
+					 booltostr(node->istemp));
+	appendStringInfo(str, " :alias ");
+	_outNode(str, node->alias);
 }
 
 static void
-_outAttr(StringInfo str, Attr *node)
+_outColumnRef(StringInfo str, ColumnRef *node)
 {
-	appendStringInfo(str, " ATTR :relname ");
-	_outToken(str, node->relname);
-	appendStringInfo(str, " :attrs ");
-	_outNode(str, node->attrs);
+	appendStringInfo(str, " COLUMNREF :fields ");
+	_outNode(str, node->fields);
+	appendStringInfo(str, " :indirection ");
+	_outNode(str, node->indirection);
+}
+
+static void
+_outParamRef(StringInfo str, ParamRef *node)
+{
+	appendStringInfo(str, " PARAMREF :number %d :fields ", node->number);
+	_outNode(str, node->fields);
+	appendStringInfo(str, " :indirection ");
+	_outNode(str, node->indirection);
 }
 
 static void
@@ -1322,6 +1373,17 @@ _outAConst(StringInfo str, A_Const *node)
 	_outNode(str, node->typename);
 }
 
+static void
+_outExprFieldSelect(StringInfo str, ExprFieldSelect *node)
+{
+	appendStringInfo(str, " EXPRFIELDSELECT :arg ");
+	_outNode(str, node->arg);
+	appendStringInfo(str, " :fields ");
+	_outNode(str, node->fields);
+	appendStringInfo(str, " :indirection ");
+	_outNode(str, node->indirection);
+}
+
 static void
 _outConstraint(StringInfo str, Constraint *node)
 {
@@ -1370,18 +1432,19 @@ _outFkConstraint(StringInfo str, FkConstraint *node)
 {
 	appendStringInfo(str, " FKCONSTRAINT :constr_name ");
 	_outToken(str, node->constr_name);
-	appendStringInfo(str, " :pktable_name ");
-	_outToken(str, node->pktable_name);
+	appendStringInfo(str, " :pktable ");
+	_outNode(str, node->pktable);
 	appendStringInfo(str, " :fk_attrs ");
 	_outNode(str, node->fk_attrs);
 	appendStringInfo(str, " :pk_attrs ");
 	_outNode(str, node->pk_attrs);
-	appendStringInfo(str, " :match_type ");
-	_outToken(str, node->match_type);
-	appendStringInfo(str, " :actions %d :deferrable %s :initdeferred %s",
-					 node->actions,
+	appendStringInfo(str, " :fk_matchtype %c :fk_upd_action %c :fk_del_action %c :deferrable %s :initdeferred %s :skip_validation %s",
+					 node->fk_matchtype,
+					 node->fk_upd_action,
+					 node->fk_del_action,
 					 booltostr(node->deferrable),
-					 booltostr(node->initdeferred));
+					 booltostr(node->initdeferred),
+					 booltostr(node->skip_validation));
 }
 
 static void
@@ -1416,7 +1479,6 @@ _outNullTest(StringInfo str, NullTest *node)
 {
 	appendStringInfo(str, " NULLTEST :arg ");
 	_outNode(str, node->arg);
-
 	appendStringInfo(str, " :nulltesttype %d ",
 					 (int) node->nulltesttype);
 }
@@ -1429,11 +1491,25 @@ _outBooleanTest(StringInfo str, BooleanTest *node)
 {
 	appendStringInfo(str, " BOOLEANTEST :arg ");
 	_outNode(str, node->arg);
-
 	appendStringInfo(str, " :booltesttype %d ",
 					 (int) node->booltesttype);
 }
 
+/*
+ *	ConstraintTest
+ */
+static void
+_outConstraintTest(StringInfo str, ConstraintTest *node)
+{
+	appendStringInfo(str, " CONSTRAINTTEST :arg ");
+	_outNode(str, node->arg);
+	appendStringInfo(str, " :testtype %d :name ",
+					 (int) node->testtype);
+	_outToken(str, node->name);
+	appendStringInfo(str, " :check_expr ");
+	_outNode(str, node->check_expr);
+}
+
 /*
  * _outNode -
  *	  converts a Node into ascii string and append it to 'str'
@@ -1476,6 +1552,12 @@ _outNode(StringInfo str, void *obj)
 			case T_IndexStmt:
 				_outIndexStmt(str, obj);
 				break;
+			case T_NotifyStmt:
+				_outNotifyStmt(str, obj);
+				break;
+			case T_SelectStmt:
+				_outSelectStmt(str, obj);
+				break;
 			case T_ColumnDef:
 				_outColumnDef(str, obj);
 				break;
@@ -1536,6 +1618,9 @@ _outNode(StringInfo str, void *obj)
 			case T_SubqueryScan:
 				_outSubqueryScan(str, obj);
 				break;
+			case T_FunctionScan:
+				_outFunctionScan(str, obj);
+				break;
 			case T_Material:
 				_outMaterial(str, obj);
 				break;
@@ -1614,6 +1699,9 @@ _outNode(StringInfo str, void *obj)
 			case T_TargetEntry:
 				_outTargetEntry(str, obj);
 				break;
+			case T_Alias:
+				_outAlias(str, obj);
+				break;
 			case T_RangeTblEntry:
 				_outRangeTblEntry(str, obj);
 				break;
@@ -1647,21 +1735,24 @@ _outNode(StringInfo str, void *obj)
 			case T_JoinInfo:
 				_outJoinInfo(str, obj);
 				break;
-			case T_Iter:
-				_outIter(str, obj);
-				break;
-			case T_Stream:
-				_outStream(str, obj);
-				break;
 			case T_A_Expr:
 				_outAExpr(str, obj);
 				break;
-			case T_Ident:
-				_outIdent(str, obj);
+			case T_RangeVar:
+				_outRangeVar(str, obj);
+				break;
+			case T_ColumnRef:
+				_outColumnRef(str, obj);
+				break;
+			case T_ParamRef:
+				_outParamRef(str, obj);
 				break;
 			case T_A_Const:
 				_outAConst(str, obj);
 				break;
+			case T_ExprFieldSelect:
+				_outExprFieldSelect(str, obj);
+				break;
 			case T_Constraint:
 				_outConstraint(str, obj);
 				break;
@@ -1680,20 +1771,15 @@ _outNode(StringInfo str, void *obj)
 			case T_BooleanTest:
 				_outBooleanTest(str, obj);
 				break;
-			case T_VariableSetStmt:
-				break;
-			case T_SelectStmt:
-				_outSelectStmt(str, obj);
+			case T_ConstraintTest:
+				_outConstraintTest(str, obj);
 				break;
 			case T_FuncCall:
 				_outFuncCall(str, obj);
 				break;
-			case T_Attr:
-				_outAttr(str, obj);
-				break;
 
 			default:
-				elog(NOTICE, "_outNode: don't know how to print type %d ",
+				elog(WARNING, "_outNode: don't know how to print type %d ",
 					 nodeTag(obj));
 				break;
 		}
