@@ -7,11 +7,11 @@
  * the nature and use of path keys.
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/optimizer/path/pathkeys.c,v 1.36.2.1 2005/01/23 02:25:30 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/optimizer/path/pathkeys.c,v 1.41 2002/09/18 21:35:21 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -518,7 +518,9 @@ build_index_pathkeys(Query *root,
 		List	   *funcargs = NIL;
 
 		funcnode->funcid = index->indproc;
-		funcnode->functype = get_func_rettype(index->indproc);
+		funcnode->funcresulttype = get_func_rettype(index->indproc);
+		funcnode->funcretset = false;	/* can never be a set */
+		funcnode->funcformat = COERCE_DONTCARE;	/* to match any user expr */
 		funcnode->func_fcache = NULL;
 
 		while (*indexkeys != 0)
@@ -621,12 +623,7 @@ find_indexkey_var(Query *root, RelOptInfo *rel, AttrNumber varattno)
  *	  vars they were joined with; furthermore, it doesn't matter what kind
  *	  of join algorithm is actually used.
  *
- *	  EXCEPTION: in a FULL or RIGHT join, we cannot treat the result as
- *	  having the outer path's path keys, because null lefthand rows may be
- *	  inserted at random points.  It must be treated as unsorted.
- *
  * 'joinrel' is the join relation that paths are being formed for
- * 'jointype' is the join type (inner, left, full, etc)
  * 'outer_pathkeys' is the list of the current outer path's path keys
  *
  * Returns the list of new path keys.
@@ -634,12 +631,8 @@ find_indexkey_var(Query *root, RelOptInfo *rel, AttrNumber varattno)
 List *
 build_join_pathkeys(Query *root,
 					RelOptInfo *joinrel,
-					JoinType jointype,
 					List *outer_pathkeys)
 {
-	if (jointype == JOIN_FULL || jointype == JOIN_RIGHT)
-		return NIL;
-
 	/*
 	 * This used to be quite a complex bit of code, but now that all
 	 * pathkey sublists start out life canonicalized, we don't have to do
@@ -777,22 +770,22 @@ find_mergeclauses_for_pathkeys(Query *root,
 
 		/*
 		 * We can match a pathkey against either left or right side of any
-		 * mergejoin clause.  (We examine both sides since we aren't told if
-		 * the given pathkeys are for inner or outer input path; no confusion
-		 * is possible.)  Furthermore, if there are multiple matching
-		 * clauses, take them all.  In plain inner-join scenarios we expect
-		 * only one match, because redundant-mergeclause elimination will
-		 * have removed any redundant mergeclauses from the input list.
-		 * However, in outer-join scenarios there might be multiple matches.
-		 * An example is
+		 * mergejoin clause.  (We examine both sides since we aren't told
+		 * if the given pathkeys are for inner or outer input path; no
+		 * confusion is possible.)	Furthermore, if there are multiple
+		 * matching clauses, take them all.  In plain inner-join scenarios
+		 * we expect only one match, because redundant-mergeclause
+		 * elimination will have removed any redundant mergeclauses from
+		 * the input list. However, in outer-join scenarios there might be
+		 * multiple matches. An example is
 		 *
-		 *		select * from a full join b on
-		 *		a.v1 = b.v1 and a.v2 = b.v2 and a.v1 = b.v2;
+		 * select * from a full join b on a.v1 = b.v1 and a.v2 = b.v2 and
+		 * a.v1 = b.v2;
 		 *
-		 * Given the pathkeys ((a.v1), (a.v2)) it is okay to return all
-		 * three clauses (in the order a.v1=b.v1, a.v1=b.v2, a.v2=b.v2)
-		 * and indeed we *must* do so or we will be unable to form a
-		 * valid plan.
+		 * Given the pathkeys ((a.v1), (a.v2)) it is okay to return all three
+		 * clauses (in the order a.v1=b.v1, a.v1=b.v2, a.v2=b.v2) and
+		 * indeed we *must* do so or we will be unable to form a valid
+		 * plan.
 		 */
 		foreach(j, restrictinfos)
 		{
@@ -820,8 +813,8 @@ find_mergeclauses_for_pathkeys(Query *root,
 			break;
 
 		/*
-		 * If we did find usable mergeclause(s) for this sort-key position,
-		 * add them to result list.
+		 * If we did find usable mergeclause(s) for this sort-key
+		 * position, add them to result list.
 		 */
 		mergeclauses = nconc(mergeclauses, matched_restrictinfos);
 	}
@@ -863,7 +856,8 @@ make_pathkeys_for_mergeclauses(Query *root,
 		cache_mergeclause_pathkeys(root, restrictinfo);
 
 		key = (Node *) get_leftop(restrictinfo->clause);
-		if (IsA(key, Var) &&intMember(((Var *) key)->varno, rel->relids))
+		if (IsA(key, Var) &&
+			VARISRELMEMBER(((Var *) key)->varno, rel))
 		{
 			/* Rel is left side of mergeclause */
 			pathkey = restrictinfo->left_pathkey;
@@ -871,7 +865,8 @@ make_pathkeys_for_mergeclauses(Query *root,
 		else
 		{
 			key = (Node *) get_rightop(restrictinfo->clause);
-			if (IsA(key, Var) &&intMember(((Var *) key)->varno, rel->relids))
+			if (IsA(key, Var) &&
+				VARISRELMEMBER(((Var *) key)->varno, rel))
 			{
 				/* Rel is right side of mergeclause */
 				pathkey = restrictinfo->right_pathkey;
